==> heptapod-rails-0.24.0rc1.tar.gz: heptapod-rails-0.24.0rc1/spec/lib/heptapod/native_mercurial/migrator_spec.rb <==
# frozen_string_literal: true

require 'spec_helper'

RSpec.describe Heptapod::NativeMercurial::Migrator do
  let_it_be(:project) { create(:hg_git_project, :hg_repository)}
  let_it_be(:repo) { project.repository }
  let_it_be(:user) { build(:user) }
  let_it_be(:prepare_repo) do
    repo.hg_call(%w[up 0])
    hg_commit(1)
    repo.hg_call(%w[up 0])
    hg_commit(2, topic: 'feature')
  end

  let(:hg_sha) { repo.hgsha_from_rev('feature') }
  let(:git_sha) { repo.sha_from_hgsha(hg_sha) }

  subject do
    project.reload
    described_class.new(project)
  end

  def hg_commit(cid, topic: nil)
    if topic
      repo.hg_call(['topic', topic])
    end

    repo.write_file(repo.hg_full_path + "/file#{cid}", "line\n")
    repo.hg_call(["add", "file#{cid}"])
    repo.hg_call(['ci', '-m', "Commit #{cid}", '--config',
                  'experimental.single-head-per-branch=False'],
                 user: user, for_write: true)
    repo.hg_git_invalidate_maps!
  end

  before(:context) do
    prepare_repo
  end

  context 'Issues' do
    let!(:issue) do
      create(:issue,
        project: project,
        description: "Sample description with Git commit: #{git_sha}",
        title: "Sample title for Git commit: #{git_sha}")
    end

    it 'migrate title and description' do
      subject.migrate_issues
      issue.reload
      expect(issue.description).to eq("Sample description with Git commit: #{hg_sha}")
      expect(issue.title).to eq("Sample title for Git commit: #{hg_sha}")
    end

    context 'Notes' do
      let!(:my_comment) do
        create(:note_on_issue, project: project, noteable: issue,
          note: "Sample comment with Git commit: #{git_sha}")
      end

      it 'migrate comments' do
        subject.migrate_notes
        my_comment.reload
        expect(my_comment.note).to eq("Sample comment with Git commit: #{hg_sha}")
      end
    end
  end

  context 'Merge Requests' do
    let!(:merge_request) do
      create(:merge_request,
        source_project: project,
        source_branch: 'topic/default/feature',
        target_project: project,
        target_branch: 'branch/default',
        description: "Sample description with Git commit: #{git_sha}",
        title: "Sample title for Git commit: #{git_sha}")
    end

    it 'migrate title and description' do
      subject.migrate_merge_requests
      merge_request.reload
      expect(merge_request.description).to eq("Sample description with Git commit: #{hg_sha}")
      expect(merge_request.title).to eq("Sample title for Git commit: #{hg_sha}")
    end

    context 'Notes' do
      let!(:my_comment) do
        create(:note_on_merge_request, project: project, noteable: merge_request,
          note: "Sample comment with Git commit: #{git_sha}")
      end

      it 'migrate comments' do
        subject.migrate_notes
        my_comment.reload
        expect(my_comment.note).to eq("Sample comment with Git commit: #{hg_sha}")
      end
    end
  end

  context 'Releases' do
    let!(:release) do
      create(:release,
        project: project,
        description: "Sample description with Git commit: #{git_sha}")
    end

    it 'migrate description' do
      subject.migrate_releases
      release.reload
      expect(release.description).to eq("Sample description with Git commit: #{hg_sha}")
    end
  end
end

==> heptapod-rails-0.25.2.tar.gz: heptapod-rails-0.25.2/spec/lib/heptapod/native_mercurial/migrator_spec.rb <==
# frozen_string_literal: true

require 'spec_helper'

RSpec.describe Heptapod::NativeMercurial::Migrator do
  let(:project) { create(:hg_git_project, :hg_repository)}
  let(:repo) { project.repository }
  let(:user) { build(:user) }
  let(:prepare_repo) do
    repo.hg_call(%w[up 0])
    hg_commit(1)
    repo.hg_call(%w[up 0])
    hg_commit(2, topic: 'feature')
  end

  let(:hg_sha) { repo.hgsha_from_rev('feature') }
  let(:git_sha) { repo.sha_from_hgsha(hg_sha) }

  subject do
    described_class.new(project)
  end

  def hg_commit(cid, topic: nil)
    if topic
      repo.hg_call(['topic', topic])
    end

    repo.write_file(repo.hg_full_path + "/file#{cid}", "line\n")
    repo.hg_call(["add", "file#{cid}"])
    repo.hg_call(['ci', '-m', "Commit #{cid}", '--config',
                  'experimental.single-head-per-branch=False'],
                 user: user, for_write: true)
    repo.hg_git_invalidate_maps!
  end

  before do
    prepare_repo
  end

  context 'Issues' do
    let!(:issue) do
      create(:issue,
        project: project,
        description: "Sample description with Git commit: #{git_sha}",
        title: "Sample title for Git commit: #{git_sha}")
    end

    it 'migrate title and description' do
      subject.migrate_issues
      issue.reload
      expect(issue.description).to eq("Sample description with Git commit: #{hg_sha}")
      expect(issue.title).to eq("Sample title for Git commit: #{hg_sha}")
    end

    context 'Notes' do
      let!(:my_comment) do
        create(:note_on_issue, project: project, noteable: issue,
          note: "Sample comment with Git commit: #{git_sha}")
      end

      it 'migrate comments' do
        subject.migrate_notes
        my_comment.reload
        expect(my_comment.note).to eq("Sample comment with Git commit: #{hg_sha}")
      end
    end
  end

  context 'Merge Requests' do
    let!(:merge_request) do
      create(:merge_request,
        source_project: project,
        source_branch: 'topic/default/feature',
        target_project: project,
        target_branch: 'branch/default',
        description: "Sample description with Git commit: #{git_sha}",
        title: "Sample title for Git commit: #{git_sha}")
    end

    it 'migrate title and description' do
      subject.migrate_merge_requests
      merge_request.reload
      expect(merge_request.description).to eq("Sample description with Git commit: #{hg_sha}")
      expect(merge_request.title).to eq("Sample title for Git commit: #{hg_sha}")
    end

    context 'Notes' do
      let!(:my_comment) do
        create(:note_on_merge_request, project: project, noteable: merge_request,
          note: "Sample comment with Git commit: #{git_sha}")
      end

      it 'migrate comments' do
        subject.migrate_notes
        my_comment.reload
        expect(my_comment.note).to eq("Sample comment with Git commit: #{hg_sha}")
      end
    end
  end

  context 'Releases' do
    let!(:release) do
      create(:release,
        project: project,
        description: "Sample description with Git commit: #{git_sha}")
    end

    it 'migrate description' do
      subject.migrate_releases
      release.reload
      expect(release.description).to eq("Sample description with Git commit: #{hg_sha}")
    end
  end

  describe '#vcs_type_enforcer' do
    let!(:project) { create(:native_hg_project, :hg_repository) }

    subject { described_class.new(project) }

    it 'force repository to be HgGitRepository' do
      expect(project.vcs_type).to eq("hg")
      subject.vcs_type_enforcer do
        expect(project.vcs_type).to eq("hg_git")
      end
      expect(project.vcs_type).to eq("hg")
    end
  end
end
